# -*- coding: utf-8 -*-
import numpy
import pylab
import matplotlib.pyplot as plt
import matplotlib.colors
import itertools
import re
import hashlib
from string import ascii_lowercase
from concho.config import Configurator
from concho.config import Config
from concho.config import Cpu
from concho import dell

markerTypes=[',', '+', '.', '^', 'v', '<', '>', 'o', '*', '1', '2', '3', '4', '8', 's', 'p', 'h', 'H', 'x', 'X', 'D', 'd', '|', '_']
#for c in ascii_lowercase:
#    markerTypes.append('$%s$' % c)
#markerColors=('r', 'g', 'b')
markerColors=('r')

def get_marker(proc_id):
    hash_object = hashlib.md5(proc_id.encode('utf-8'))
    hash = int(hash_object.hexdigest(), 16)
    return markerTypes[ hash % len(markerTypes) ]

def plotCpuPassmark():
    cpuTable = numpy.genfromtxt('cpu_table.dat', dtype=("|U10", float, int, float, float), names=True, delimiter='\t')
    plt.subplot(1,1,0)
    plt.subplots_adjust(bottom = 0.1)
    markersCycler = itertools.cycle(itertools.product(markerTypes, markerColors))
    labels = cpuTable['id']
    x = cpuTable['clock'] * cpuTable['num_cores']
    y = cpuTable['cpumark']
    markerSize = 50
    color = 'b'
    for label, x1, y1 in zip(labels, x, y):
        if  y1 <= 0.0:
            continue # no passmark available fo this data
        generation=label[-1]
        if generation == '2':
            color = 'b'
        else:
            color = 'r'            
        marker = markersCycler.next()
        plt.scatter( x1, y1, color = color, s = markerSize, marker = marker[0], label = label)    
    plt.xlabel(u'theoretical cpu speed [core.GHz]')
    plt.ylabel(u'passmark [?]')
    plt.title(u'comparison between cpu theoretical and effective speed')
    plt.xlim( xmin = 0.0 )
    plt.ylim( ymin = 0.0 )
    plt.legend(bbox_to_anchor=(0.2, 1.0))
    #plt.legend()
    plt.draw()
    plt.show()


def create_unique_marker(config_index):
    marker_string=''
    alphabet_size = 26
    alphabel_base = 0x41
    while True:
        digit_index = config_index % alphabet_size
        marker_string = chr(alphabel_base + digit_index) + marker_string
        if config_index < alphabet_size:
            break
        else:
            config_index = config_index // alphabet_size
    return marker_string

def plot_system_efficiency():

    cpuTable = numpy.genfromtxt('cpu_table.dat', dtype=("|U15", float, int, float, float, float), names=True, delimiter='\t')
    #cpuTable = numpy.genfromtxt('dell_ivybridge_table.dat', dtype=(('id', "|S10"), ('clock', float), ('num_cores', int), ('price', float, float)), names=None, delimiter='\t')
    print(type(cpuTable))
    print(cpuTable.dtype)
    print(cpuTable)
    print(cpuTable['id'])

    #cpuTable = numpy.genfromtxt('dell_ivybridge_table.dat', dtype=(('id', "|S10"), ('clock', float), ('num_cores', int), ('price', float, float)), names=None, delimiter='\t')

    #for (x, y) in clusters:
    def GHzToMHz( frequency ):
        return frequency * 1000.0
        
    kWHPrice = 0.07 * 1.5
    containerLifetime = 7.0 # in years
    powerUsageEfficiency = 0.5


    def getColorCodeFromItemLabel(label):
        generation=label[-1]
        (num_servers, model, proc_id, ram_size) = re.split('_', label)
        saturation = {
            'sandy bridge':0.0,
            'ivy bridge':0.2,
            'haswell':0.2,
            'broadwell':0.2,
            'skylake':0.4,
            'coffeelake':0.6,
            'cascadelake':1.0
            }[Cpu(proc_id).architecture]
            #         if model == 'r620':
            #             color = 'r'
            #         elif model == 'r630':
            #             color = 'g'
            #         elif model == 'r730':
            #             color = 'm'
            #         elif model == 'c6220':
            #             if generation == '2':
            #                 color = 'b'
            #             else:
            #                 color = 'y'
        hue = {
            'dell-poweredge-r620': 0.6,
            'dell-poweredge-r630': 0.6,
            'dell-poweredge-r640': 0.6,
            'dell-poweredge-c4310': 0.6,
            'dell-poweredge-r730': 0.4,
            'dell-poweredge-r940': 0.8,
            'dell-poweredge-c6220': 1.0,
            'dell-poweredge-c6320': 1.0,
            'dell-poweredge-c6420': 1.0,
            'dell-precision-3630': 0.2
            }[model]
        value = 0.9
        return matplotlib.colors.hsv_to_rgb((hue, saturation, value))

    def get_marker_from_label(label):
        (model, proc_id, ram_size) = re.split('_', label)
        return get_marker(proc_id)


    item_price = numpy.array([])
    item_power_consumption = numpy.array([])
    item_speed = numpy.array([])
    item_label = numpy.array([])
    configurators = [
        dell.DellMatinfoCsvConfigurator('c6420-20200716-price.tsv'),
        dell.DellMatinfoConfigurator('rcrc1406676-4834664 - Cat2 Conf4 PowerEdge R640 - Dell.html'),
        dell.DellMatinfoConfigurator('rcrc1406676-4824727 - Cat 2 Conf 7 PowerEdge R940 - Dell.html'),
        # dell.DellPowerEdgeR940(),
        ]
    for configurator in configurators:
        for cpu in configurator.get_cpu_options():
            if not cpu.architecture in ['coffeelake', 'skylake','cascadelake']:
                continue
            config = configurator.create_config()
            config.num_cpu_per_server = config.configurator.chassis.item.num_cpu_slots_per_server
            config.set_cpu(cpu)
            config.set_ram(ram_per_core=4.0e9)

            config_label = str(config.num_servers) + '_' + config.chassis.uid + '_' + cpu.uid + '_' + str(config.ram_size/config.num_servers) + 'gb'
            item_label = numpy.append( item_label, config_label)
            # print('procOptionPrice', procOptionPrice)
            # config_price = procOptionPrice
            # config_price += config.get_empty_price()
            # print('config.get_empty_price()', config.get_empty_price())
            # ram_update_price = config.get_ram_update_price(cpu=cpu, ram_per_core=4.0e9)
            # # ram_update_price = config.get_ram_update_price(cpu=cpu, ram_per_server=192.0e9)
            # # ram_update_price = config.get_ram_update_price(cpu=cpu, ram_per_cpu=96.0e9)
            # print('ram_update_price', ram_update_price)
            # config_price += ram_update_price
            # config_price += config.get_guarantee_price(5)
            # print('config.config.get_guarantee_price(5)', config.get_guarantee_price(5))
            # config_price += config.get_disk_upgrade_price(2.0e12)
            # print('config.get_disk_upgrade_price(2.0e12)', config.get_disk_upgrade_price(2.0e12))
            # print(item_label, config.get_price(), config.get_power_consumption())
            item_price = numpy.append( item_price, config.get_price() )
            item_power_consumption = numpy.append( item_power_consumption, config.get_power_consumption())
            # # print(hostTypeId, procId, item_power_consumption[-1])
            item_speed = numpy.append( item_speed, config.get_flops())

    #pylab.plot(x, y, '+')
    #pylab.xlabel('speed/price ratio [core.MHz/euros]')
    #pylab.ylabel('speed/power consumption ratio [core.MHz/W]')
    #pylab.show() # or savefig(<filename>)

    
    #print("items = ")
    #print(item_label)

    markerSize = 50

    if False:
        plt.subplot(1,2,1)
        plt.subplots_adjust(bottom = 0.1)
        markersCycler = itertools.cycle(itertools.product(markerTypes, markerColors))
        x = item_speed / item_price
        y = item_speed / item_power_consumption
        for label, x1, y1, power, speed, price, in zip(item_label, x, y, item_power_consumption, item_speed, item_price):
            marker = markersCycler.next()
            color = getColorCodeFromItemLabel(label)
            plt.scatter( x1, y1, color = color, s = markerSize, marker = marker[0], label = label)
            #print(x1, y1, color, markerSize, marker[0], label)
        if False:
            plt.scatter( x, y, marker = 'o')
            for label, x1, y1, power, speed, price, in zip(item_label, x, y, item_power_consumption, item_speed, item_price):
                #print(label)
                plt.annotate( u'%s (%.1f core.GHz, %.0f W, %.0f €)' % (label,speed/1000.0, power, price), 
                    xy = (x1, y1), xytext = (-50, 50),
                    textcoords = 'offset points', ha = 'right', va = 'bottom',
                    bbox = dict(boxstyle = 'round,pad=0.5', fc = 'yellow', alpha = 0.5),
                    arrowprops = dict(arrowstyle = '->', connectionstyle = 'arc3,rad=0'))
        plt.xlabel(u'speed/price ratio [core.MHz/€]')
        plt.ylabel(u'speed/power consumption ratio [core.MHz/W]')
        plt.xlim( xmin = 0.0 )
        plt.ylim( ymin = 0.0 )

    plt.subplot(1,2,1)
    #fig = plt.figure()
    #ax = fig.gca()
    #ax.set_xticks(numpy.arange(0,1,0.1))
    #ax.set_yticks(numpy.arange(0,1.,0.1))

    powerUsedInLifetime = (item_power_consumption * containerLifetime * 365 * 24) / powerUsageEfficiency
    itemTotalCost = item_price + (powerUsedInLifetime / 1000.0 * kWHPrice )
    markersCycler = itertools.cycle(itertools.product(markerTypes, markerColors))
    item_flops = item_speed
    # print item_flops
    item_total_num_ops = item_flops * containerLifetime * 365 * 24 * 3600
    print(item_price)
    x = item_price
    y = item_total_num_ops / itemTotalCost
    for i in range(len(item_label)):
        print(item_label[i], item_price[i], y[i])
        print('itemTotalCost', itemTotalCost[i])
        print('flops', item_flops[i])
    # print y
    config_index = 0
    for label, x1, y1, power, speed, price, in zip(item_label, x, y, item_power_consumption, item_speed, item_price):
        print(label, x1, y1)
        if y1 > 0.0001:
            color = getColorCodeFromItemLabel(label)
            # marker = markersCycler.next()
            # marker = get_marker_from_label( label )
            #print(x1, y1)
            short_label = label.replace('dell-poweredge-','').replace('intel-xeon-','')
            plt.scatter( x1, y1, facecolors=color, s=(markerSize * len(create_unique_marker(config_index))) , marker='$\mathrm{%s}$' % create_unique_marker(config_index), label=short_label)
            if False: # y1 > 5.0e16:
                plt.annotate( u'%s' % short_label, 
                    xy = (x1, y1), xytext = (x1*4.0, (y1-5.2e16)*7.1),
                    textcoords = 'data', ha = 'right', va = 'bottom',
                    bbox = dict(boxstyle = 'round,pad=0.5', fc = 'yellow', alpha = 0.5),
                    arrowprops = dict(arrowstyle = '->', connectionstyle = 'arc3,rad=0'))
            config_index += 1

    plt.xlabel(u'purchase price [€]')
    plt.ylabel(u'num total DP operations/total cost [€/^-1]')
    plt.title(u'total cost including electricity')
    plt.xlim( xmin = 0.0 )
    plt.ylim( ymin = 0.0 )
    plt.minorticks_on()
    plt.grid(b=True, which='major', color='b', linestyle='-', linewidth=0.5)
    plt.grid(b=True, which='minor', color='b', linestyle='-', linewidth=0.2)    
    plt.legend(bbox_to_anchor=(1.1, 1.1), ncol=3)
    plt.draw()


    plt.show()

#plotCpuPassmark():
if __name__ == '__main__':
    plot_system_efficiency()
